#!/usr/bin/perl
#
# Copyright (C) 2015  Internet Systems Consortium, Inc. ("ISC")
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
# INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
# LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
# OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.

# server-json.pl:
# Parses the JSON version of the server stats into a normalized format.

use JSON;

open(INPUT, "<json.stats");
my $text = do{local$/;<INPUT>};
close(INPUT);

my $ref = decode_json($text);
foreach $key (keys $ref->{opcodes}) {
    print "opcode " . $key . ": " . $ref->{opcodes}->{$key} . "\n";
}
foreach $key (keys $ref->{qtypes}) {
    print "qtype " . $key . ": " . $ref->{qtypes}->{$key} . "\n";
}
foreach $key (keys $ref->{nsstats}) {
    print "nsstat " . $key . ": " . $ref->{nsstats}->{$key} . "\n";
}
